"""Get EXIF data from a directory of photos"""

from pyexiv2 import Image  # type: ignore
import os
import re


def scan_tree(directory: str):
    """Recursively yield DirEntry objects for given directory."""
    for entry in os.scandir(directory):
        if entry.is_dir(follow_symlinks=False):
            yield from scan_tree(entry.path)  # see below for Python 2.x
        else:
            yield entry


def get_photos(this_directory: str) -> list:
    """Get photos from a directory.

    :param this_directory: a directory containing photos
    :returns: A list containing photo files with complete path
    """
    directory_files: list = scan_tree(this_directory)
    regex = re.compile('CR2$|jpg$')  # needs to be generalized for all possible photo extensions
    photos: list = []
    for file in directory_files:
        if regex.search(file.name) and file.is_file():
            photos.append(file.path)
    return photos


def get_exif(photo_list: list) -> list:
    """Obtain exif data for each photo from the list.

    :param photo_list: A list of photos from a directory
    :returns: A list of exif dictionaries from photos
    """
    exif_list = []
    for image in photo_list:
        this_image = Image(image)
        try:
            this_image_exif = this_image.read_exif()
        except UnicodeDecodeError:
            print(f"Could not get exif data for {image}")
        exif_list.append(this_image_exif)
    return exif_list



import argparse

import get_exif  # type: ignore
import lenses  # type: ignore


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("-p", "--path", help="Path to recursively search for photos")
    parser.add_argument("-g", "--graphpath", help="Path to save graphs generated by this program")
    return parser.parse_args()


def main():
    args = parse_args()
    photos = get_exif.get_photos(args.path)
    exif = get_exif.get_exif(photos)
    lenses.main(exif, args.graphpath)


if __name__ == '__main__':
    main()



"""Create a graphs relating to lens usage on a directory of photos."""

from collections import Counter
import matplotlib.pyplot as plt  # type: ignore

from photostats import get_exif
from photostats.utils import create_plot

def get_lens_make_model(exif_list: list) -> dict:
    """Obtain lens make and model for each photo passed in.

    :param exif_list: A list of photo files.
    :returns: A Counter dictionary, counting the makes/models of lenses.
    """
    lens_list = []
    for data in exif_list:
        if data.get("Exif.Photo.LensModel") is None:
            pass
        else:
            lens_list.append((data.get("Exif.Photo.LensModel")))
    return Counter(lens_list)


def get_focal_length(exif_list: list) -> dict:
    """Obtain focal length for each photo passed in.

        :param exif_list: A list of photo files.
        :returns: A Counter dictionary, counting the focal length of lenses.
        """
    focal_length_list = []
    for data in exif_list:
        pre_fixed_string = f"{data.get('Exif.Photo.FocalLength')}"
        fixed_string = f"{pre_fixed_string[:pre_fixed_string.find('/')]}"
        try:
            if int(fixed_string) > 1000:
                pass
            else:
                focal_length_list.append(fixed_string)
        except ValueError:
            pass
    return Counter(sorted(focal_length_list, key=int))


def main(exif, graph_path):
    lens_count = get_lens_make_model(exif)
    print("Lens Model Count:")
    for lens, count in lens_count.items():
        print(f'{lens} : {count}')
    create_plot.create_plot(lens_count.keys(), lens_count.values(), x_label="Lens Model", y_label="Number of Photos",
                            title="Number of Photos Taken by Each Lens", graph_path=graph_path,
                            graph_filename="lens_models")
    print("\nFocal Length Count:")
    focal_length_count = get_focal_length(exif)
    for focal_length, count in focal_length_count.items():
        print(f'{focal_length} mm : {count}')
    create_plot.create_plot(focal_length_count.keys(), focal_length_count.values(), x_label="Focal Length (mm)",
                            y_label="Number of Photos", title="Number of Photos Taken at Each Focal Length",
                            graph_path=graph_path, graph_filename="lens_focal_length")


if __name__ == '__main__':
    test_directory = "/media/Photos/My Photos 2005 and on/2020/"
    test_graph_path = "/home/ermesa/tmp/photostats"
    test_photos = get_exif.get_photos(test_directory)
    test_exif = get_exif.get_exif(test_photos)
    main(test_exif, test_graph_path)



"""Create a plot and save to disk."""

import matplotlib.pyplot as plt  # type: ignore


def create_plot(group_names: list, group_data: list, x_label: str = "", y_label: str = "", title: str = "",
                graph_path: str = "", graph_filename: str = ""):
    plt.style.use('fivethirtyeight')
    plt.rcParams.update({'figure.autolayout': True})
    fig, ax = plt.subplots(figsize=(8, 8))
    ax.bar(group_names, group_data)
    x_labels = ax.get_xticklabels()
    plt.setp(x_labels, rotation=30, horizontalalignment='right')
    ax.set(xlabel=x_label, ylabel=y_label, title=title)
    ax.title.set(y=1.05)
    fig.savefig(f"{graph_path}/{graph_filename}", transparent=False, dpi=80, bbox_inches="tight")
    # plt.show()
